@@ -12,13 +12,6 @@ Tomas gren <stric@ing.umu.se>
 #include "thinice_rc_style.h"
 #include "thinice_misc.h"
 
-#ifndef max
-#define max(x,y) ((x)>=(y)?(x):(y))
-#endif
-#ifndef min
-#define min(x,y) ((x)<=(y)?(x):(y))
-#endif
-
 #define DRAW_ARGS    GtkStyle       *style, \
                      GdkWindow      *window, \
                      GtkStateType    state_type, \
@@ -36,20 +29,6 @@ Tomas gren <stric@ing.umu.se>
 static void thinice_style_init       (ThiniceStyle      *style);
 static void thinice_style_class_init (ThiniceStyleClass *klass);
 
-static void
-thinice_tab(GtkStyle * style,
-            GdkWindow * window,
-            GtkStateType state_type,
-            GtkShadowType shadow_type,
-            GdkRectangle * area,
-            GtkWidget * widget,
-            const gchar *detail,
-            gint x,
-            gint y,
-            gint width,
-            gint height,
-	    gint orientation);
-
 static GtkStyleClass *thinice_parent_style_class = NULL;
 
 static void
@@ -63,13 +42,18 @@ thinice_style_draw_hline(GtkStyle * style,
            gint x2,
            gint y)
 {
+	ThiniceStyle *thinice_style = THINICE_STYLE (style);
+	CairoColor *dark;
+        CairoColor *light;
 	cairo_t *cr;
 	
 	CHECK_ARGS
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 	
-	thinice_draw_separator(cr, TRUE, x1, y, x2-x1, 2);
+	dark = &thinice_style->color_cube.dark[state_type];
+	light = &thinice_style->color_cube.light[state_type];
+	thinice_draw_separator(cr, dark, light, TRUE, x1, y, x2-x1, 2);
 	cairo_destroy (cr);
 }
 
@@ -84,6 +68,9 @@ thinice_style_draw_vline(GtkStyle * style,
            gint y2,
            gint x)
 {
+	ThiniceStyle *thinice_style = THINICE_STYLE (style);
+	CairoColor *dark;
+	CairoColor *light;
 	cairo_t *cr;
 
 	CHECK_ARGS
@@ -93,7 +80,9 @@ thinice_style_draw_vline(GtkStyle * style,
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 	
-	thinice_draw_separator(cr, FALSE, x, y1, 2, y2-y1);
+	dark = &thinice_style->color_cube.dark[state_type];
+	light = &thinice_style->color_cube.light[state_type];
+	thinice_draw_separator(cr, dark, light, FALSE, x, y1, 2, y2-y1);
 
 	cairo_destroy (cr);
 }
@@ -151,13 +140,11 @@ thinice_style_draw_shadow(GtkStyle     *style,
 
     case GTK_SHADOW_ETCHED_IN:
     case GTK_SHADOW_ETCHED_OUT:
-	  ge_cairo_set_color(cr, color1);	
-   	  cairo_rectangle(cr, x + 1.5, y + 1.5, width - 3, height - 3);  
-	  cairo_stroke(cr);
+	  ge_cairo_set_color(cr, color1);
+   	  ge_cairo_stroke_rectangle(cr, x + 1.5, y + 1.5, width - 3, height - 3);  
 
 	  ge_cairo_set_color(cr, color2);	
-   	  cairo_rectangle(cr, x + 0.5, y + 0.5, width - 3, height - 3);  
-	  cairo_stroke(cr);
+   	  ge_cairo_stroke_rectangle(cr, x + 0.5, y + 0.5, width - 3, height - 3);  
       break;
 
     case GTK_SHADOW_IN:
@@ -181,15 +168,8 @@ thinice_style_draw_polygon(GtkStyle * style,
              gint npoints,
              gint fill)
 {
-#ifndef M_PI
-#define M_PI    3.14159265358979323846
-#endif /* M_PI */
-#ifndef M_PI_4
-#define M_PI_4  0.78539816339744830962
-#endif /* M_PI_4 */
-
-  static const gdouble pi_over_4 = M_PI_4;
-  static const gdouble pi_3_over_4 = M_PI_4 * 3;
+  static const gdouble pi_over_4 = G_PI_4;
+  static const gdouble pi_3_over_4 = G_PI_4 * 3;
 
   ThiniceStyle *thinice_style = THINICE_STYLE (style);
 
@@ -535,197 +515,157 @@ thinice_style_draw_box(GtkStyle * style,
          gint height)
 {
 	ThiniceStyle *thinice_style = THINICE_STYLE (style);
+	cairo_t *canvas;
+	gboolean draw_border = TRUE;
 
-  CairoColor *light, *dark;
-  cairo_t *cr;
+	/***********************************************/
+	/* GTK Sanity Checks                           */
+	/***********************************************/
+	CHECK_ARGS
+	SANITIZE_SIZE 
 
-  CHECK_ARGS
 
-  SANITIZE_SIZE
+	/***********************************************/
+	/* GTK Special Cases - Ignored Widgets         */
+	/***********************************************/
+	if ((CHECK_DETAIL(detail, "optionmenutab")) ||
+		(CHECK_DETAIL (detail, "slider")) ||
+		(CHECK_DETAIL (detail, "buttondefault")) ||
+		(CHECK_DETAIL (detail, "bar") && 
+			((height < 1) && (width < 1))))
+	{
+		return;
+	}
 
-  if (CHECK_DETAIL(detail, "optionmenutab"))
-	return;
 
-  light = &thinice_style->color_cube.light[state_type];
-  dark = &thinice_style->color_cube.dark[state_type];
+	/***********************************************/
+	/* GTK Special Cases - adjust Size/Offset      */
+	/***********************************************/
+	if ((CHECK_DETAIL (detail, "bar")))
+	{
+		x += 1;
+		y += 1;
+		width -= 2;
+		height -= 2;
 
-  cr = ge_gdk_drawable_to_cairo (window, area);
-
- if (CHECK_DETAIL (detail, "trough"))
-    {
-           ge_cairo_set_color(cr, &thinice_style->color_cube.bg[state_type]);
-
-           cairo_rectangle(cr, x, y, width, height);
-
-           cairo_fill(cr);
-
-      thinice_style_draw_shadow(style, window, state_type, shadow_type, area, widget,
-                       detail, x, y, width, height);
+		draw_border=FALSE;
     }
-  else if (CHECK_DETAIL (detail, "slider"))
-    {
-    }
-  else if (CHECK_DETAIL (detail, "buttondefault"))
-    {
-      /* I don't want no background on default buttons..
-         Let's add that cute triangle (see below) instead... */
-    }
-  else if (CHECK_DETAIL (detail, "button"))
-    {
-      if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP (window))
-        {
-           ge_cairo_set_color(cr, &thinice_style->color_cube.bg[state_type]);
+	else if (CHECK_DETAIL (detail, "handlebox_bin"))
+	{
+		draw_border=FALSE;
+	}
 
-           cairo_rectangle(cr, x+1, y+1, width-2, height-2);
 
-           cairo_fill(cr);
+	/***********************************************/
+	/* Fill Box                                    */
+	/***********************************************/
+	if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
+	{
+		canvas = ge_gdk_drawable_to_cairo (window, area);
 
-        }
-      else
-        {
-          gtk_style_apply_default_background(style, window,
-                                             widget && !GTK_WIDGET_NO_WINDOW(widget),
-                                             state_type, area,
-                                             x, y, width, height);
-        }
-      /* Paint a triangle here instead of in "buttondefault"
-         which is drawn _behind_ the current button */
-      if (widget && GTK_WIDGET_HAS_DEFAULT (widget))
-        {
-          cairo_set_line_width (cr, 1);
+		ge_cairo_set_color(canvas, &thinice_style->color_cube.bg[state_type]);
 
-          ge_cairo_set_color(cr, &thinice_style->color_cube.bg[GTK_STATE_SELECTED]);
-          cairo_move_to(cr, x+2.5, y+2.5);
-          cairo_line_to(cr, x+10.5, y+2.5);
-          cairo_line_to(cr, x+2.5, y+10.5);
-          cairo_line_to(cr, x+2.5, y+2.5);
-	  cairo_fill(cr);
+		cairo_rectangle(canvas, x, y, width, height);
 
-          ge_cairo_set_color(cr, dark);
-	  cairo_move_to(cr, x + 2.5, y + 11);
-	  cairo_line_to(cr, x + 2.5, y + 2.5);
-  	  cairo_line_to(cr, x + 11, y + 2.5);
-	  cairo_stroke(cr);
+		cairo_fill(canvas);
 
-          cairo_set_line_width (cr, 0.5);
-          ge_cairo_set_color(cr, light);
-          cairo_move_to(cr, x+11, y+3);
-          cairo_line_to(cr, x+3, y+11);
-	  cairo_stroke(cr);
-        }
+		cairo_destroy(canvas);
+	}
+	else
+	{
+		gtk_style_apply_default_background(style, window,
+				widget && !GTK_WIDGET_NO_WINDOW(widget),
+				state_type, area,
+				x, y, width, height);
+	}
+	
+	/***********************************************/
+	/* Draw Box Border                             */
+	/***********************************************/
+	if (draw_border)
+	{
+		thinice_style_draw_shadow(style, window, state_type, shadow_type, area, widget,
+						detail, x, y, width, height);
+	}
 
-      thinice_style_draw_shadow(style, window, state_type, shadow_type, area, widget,
-                       detail, x, y, width, height);
-    }
-  else if (CHECK_DETAIL (detail, "bar"))
-    {
-      if ((height > 1) && (width > 1))
-        {
+	/***********************************************/
+	/* Draw Widget Specific Sub-Parts              */
+	/***********************************************/
+	if (CHECK_DETAIL (detail, "button"))
+	{
+		/* Paint a triangle here instead of in "buttondefault"
+			which is drawn _behind_ the current button */
+		if (widget && GE_WIDGET_HAS_DEFAULT (widget))
+		{
+			canvas = ge_gdk_drawable_to_cairo (window, area);
 
-           ge_cairo_set_color(cr, &thinice_style->color_cube.bg[state_type]);
+			ge_cairo_set_color(canvas, &thinice_style->color_cube.bg[GTK_STATE_SELECTED]);
+			cairo_move_to(canvas, x+2.5, y+2.5);
+			cairo_line_to(canvas, x+10.5, y+2.5);
+			cairo_line_to(canvas, x+2.5, y+10.5);
+			cairo_line_to(canvas, x+2.5, y+2.5);
+			cairo_fill(canvas);
 
-           cairo_rectangle(cr, x+1, y+1, width-2, height-2);
+			ge_cairo_set_color(canvas, &thinice_style->color_cube.dark[state_type]);
+			cairo_move_to(canvas, x + 2.5, y + 11);
+			cairo_line_to(canvas, x + 2.5, y + 2.5);
+			cairo_line_to(canvas, x + 11, y + 2.5);
+			cairo_stroke(canvas);
 
-           cairo_fill(cr);
+			cairo_set_line_width (canvas, 0.5);
+			ge_cairo_set_color(canvas, &thinice_style->color_cube.light[state_type]);
+			cairo_move_to(canvas, x+11, y+3);
+			cairo_line_to(canvas, x+3, y+11);
+			cairo_stroke(canvas);
 
-        }
-    }
-  else if (CHECK_DETAIL (detail, "handlebox_bin"))
-    {
-      if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
-        {
-           ge_cairo_set_color(cr, &thinice_style->color_cube.bg[state_type]);
+			cairo_destroy(canvas);
+		}
+	}
 
-           cairo_rectangle(cr, x, y, width, height);
-
-           cairo_fill(cr);
-
-        }
-      else
-        {
-          gtk_style_apply_default_background(style, window,
-                                             widget && !GTK_WIDGET_NO_WINDOW(widget),
-                                             state_type, area,
-                                             x, y, width, height);
-        }
-    }
-  else if (CHECK_DETAIL (detail, "menubar"))
-    {
-      if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
-        {
-           ge_cairo_set_color(cr, &thinice_style->color_cube.bg[state_type]);
-
-           cairo_rectangle(cr, x, y, width, height);
-
-           cairo_fill(cr);
-        }
-      else
-        {
-          gtk_style_apply_default_background(style, window,
-                                             widget && !GTK_WIDGET_NO_WINDOW(widget),
-                                             state_type, area,
-                                             x, y, width, height);
-        }
-      thinice_style_draw_shadow(style, window, state_type, shadow_type, area, widget,
-                       detail, x, y, width, height);
-    }
-  else
-    {
-
-      if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
-        {
-           ge_cairo_set_color(cr, &thinice_style->color_cube.bg[state_type]);
-
-           cairo_rectangle(cr, x, y, width, height);
-
-           cairo_fill(cr);
-        }
-      else
-        {
-          gtk_style_apply_default_background(style, window,
-                                             widget && !GTK_WIDGET_NO_WINDOW(widget),
-                                             state_type, area,
-                                             x, y, width, height);
-        }
-      thinice_style_draw_shadow(style, window, state_type, shadow_type, area, widget,
-                       detail, x, y, width, height);
-    }
-
-  if (CHECK_DETAIL(detail, "optionmenu") ||  (CHECK_DETAIL(detail, "button") && 
-       (ge_is_combo_box(widget, FALSE)) && !(ge_is_combo_box_entry(widget))))
-    {
-      GtkRequisition indicator_size;
-      GtkBorder indicator_spacing;
-      gint vline_x;
+	/* Draw Option Menus and Combo Box "Tab" The Same Way Here */
+	if (CHECK_DETAIL(detail, "optionmenu") ||  (CHECK_DETAIL(detail, "button") && 
+		(ge_is_combo_box(widget, FALSE)) && !(ge_is_combo_box_entry(widget))))
+	{
+		GtkRequisition indicator_size;
+		GtkBorder indicator_spacing;
+		CairoColor *dark;
+		CairoColor *light;
+		gint vline_x;
  
-      if (state_type != GTK_STATE_INSENSITIVE)
-        state_type = GTK_STATE_NORMAL;
+		if (state_type != GTK_STATE_INSENSITIVE)
+			state_type = GTK_STATE_NORMAL;
  
-      ge_option_menu_get_props (widget, &indicator_size, &indicator_spacing);
+		ge_option_menu_get_props (widget, &indicator_size, &indicator_spacing);
  
-      if ((!widget) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL))
-	vline_x = x + indicator_size.width + indicator_spacing.left + indicator_spacing.right;
-      else
-	vline_x = x + width - (indicator_size.width + indicator_spacing.left + 
-                               indicator_spacing.right) - style->xthickness;
+		if ((!widget) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL))
+			vline_x = x + indicator_size.width + indicator_spacing.left + indicator_spacing.right;
+		else
+			vline_x = x + width - (indicator_size.width + indicator_spacing.left + 
+						indicator_spacing.right) - style->xthickness;
 
-      thinice_draw_separator(cr, FALSE, vline_x, y + style->ythickness + 1, style->xthickness, height - 2*style->ythickness - 2);
- 
-      if ((widget) && (gtk_widget_get_direction (GTK_WIDGET (widget)) == GTK_TEXT_DIR_RTL))
-         x +=  indicator_spacing.right + style->xthickness;
-      else
-         x += width - indicator_size.width - indicator_spacing.right - style->xthickness;
- 
-      y += ((height - indicator_size.height) / 2) + 1;
- 
-      width = indicator_size.width;
-      height = indicator_size.height;
- 
-      thinice_style_draw_arrow (style, window, state_type, shadow_type, area, NULL, "optionmenu", 
-	                      GTK_ARROW_DOWN, TRUE,  x,  y,  width,  height);
-   }
+		canvas = ge_gdk_drawable_to_cairo (window, area);
 
-  cairo_destroy(cr);
+		dark = &thinice_style->color_cube.dark[state_type];
+		light = &thinice_style->color_cube.light[state_type];
+		thinice_draw_separator(canvas, dark, light, FALSE,
+                                       vline_x, y + style->ythickness + 1,
+                                       style->xthickness, height - 2*style->ythickness - 2);
+
+		cairo_destroy(canvas);
+ 
+		if ((widget) && (gtk_widget_get_direction (GTK_WIDGET (widget)) == GTK_TEXT_DIR_RTL))
+			x +=  indicator_spacing.right + style->xthickness;
+		else
+			x += width - indicator_size.width - indicator_spacing.right - style->xthickness;
+ 
+		y += ((height - indicator_size.height) / 2) + 1;
+ 
+		width = indicator_size.width;
+		height = indicator_size.height;
+ 
+		thinice_style_draw_arrow (style, window, state_type, shadow_type, area, NULL, "optionmenu", 
+						GTK_ARROW_DOWN, TRUE,  x,  y,  width,  height);
+	}
 }
 
 static void
@@ -760,9 +700,7 @@ thinice_style_draw_check(GtkStyle * style,
 	{
 	  ge_cairo_set_color(cr, &thinice_style->color_cube.dark[state_type]);	
 
-   	  cairo_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);  
-
-	  cairo_stroke(cr);
+   	  ge_cairo_stroke_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);  
 	}
 	else
 	{
@@ -833,21 +771,21 @@ thinice_style_draw_option(GtkStyle * style,
     {
     case GTK_SHADOW_ETCHED_IN:
       ge_cairo_set_color(cr, color2);
-      cairo_arc(cr, centerX + 1, centerY + 1, radius + 1, 0, 2 * M_PI);
+      cairo_arc(cr, centerX + 1, centerY + 1, radius + 1, 0, 2 * G_PI);
       cairo_stroke(cr);
 
       ge_cairo_set_color(cr, color1); 
-      cairo_arc(cr, centerX, centerY, radius + 1, 0, 2 * M_PI);
+      cairo_arc(cr, centerX, centerY, radius + 1, 0, 2 * G_PI);
       cairo_stroke(cr);
 
       break;
     case GTK_SHADOW_ETCHED_OUT:
       ge_cairo_set_color(cr, color1); 
-      cairo_arc(cr, centerX - 1, centerY - 1, radius + 1, 0, 2 * M_PI);
+      cairo_arc(cr, centerX - 1, centerY - 1, radius + 1, 0, 2 * G_PI);
       cairo_stroke(cr);
 
       ge_cairo_set_color(cr, color2);
-      cairo_arc(cr, centerX, centerY, radius + 1,  0, 2 * M_PI);
+      cairo_arc(cr, centerX, centerY, radius + 1,  0, 2 * G_PI);
       cairo_stroke(cr);
 
       break;
@@ -857,8 +795,8 @@ thinice_style_draw_option(GtkStyle * style,
       cairo_new_path (cr);
 
       cairo_move_to(cr, centerX + (radius + 2), centerY + (radius + 2));
-      cairo_line_to(cr, centerX + (radius + 2)*sin(M_PI/4.0), centerY - (radius + 2)*cos(M_PI/4.0));
-      cairo_line_to(cr, centerX - (radius + 2)*sin(M_PI/4.0), centerY + (radius + 2)*cos(M_PI/4.0));
+      cairo_line_to(cr, centerX + (radius + 2)*sin(G_PI/4.0), centerY - (radius + 2)*cos(G_PI/4.0));
+      cairo_line_to(cr, centerX - (radius + 2)*sin(G_PI/4.0), centerY + (radius + 2)*cos(G_PI/4.0));
       cairo_line_to(cr, centerX + (radius + 2), centerY + (radius + 2));
 
       cairo_close_path (cr);
@@ -867,7 +805,7 @@ thinice_style_draw_option(GtkStyle * style,
       cairo_clip (cr);
 
       ge_cairo_set_color(cr, color2);
-      cairo_arc(cr, centerX, centerY, radius + 1, 0,  2*M_PI);
+      cairo_arc(cr, centerX, centerY, radius + 1, 0,  2*G_PI);
       cairo_fill(cr);
 
       cairo_restore(cr);
@@ -875,8 +813,8 @@ thinice_style_draw_option(GtkStyle * style,
       cairo_new_path (cr);
 
       cairo_move_to(cr, centerX - (radius + 2), centerY - (radius + 2));
-      cairo_line_to(cr, centerX + (radius + 2)*sin(M_PI/4.0), centerY - (radius + 2)*cos(M_PI/4.0));
-      cairo_line_to(cr, centerX - (radius + 2)*sin(M_PI/4.0), centerY + (radius + 2)*cos(M_PI/4.0));
+      cairo_line_to(cr, centerX + (radius + 2)*sin(G_PI/4.0), centerY - (radius + 2)*cos(G_PI/4.0));
+      cairo_line_to(cr, centerX - (radius + 2)*sin(G_PI/4.0), centerY + (radius + 2)*cos(G_PI/4.0));
       cairo_line_to(cr, centerX - (radius + 2), centerY - (radius + 2));
 
       cairo_close_path (cr);
@@ -886,13 +824,13 @@ thinice_style_draw_option(GtkStyle * style,
       cairo_clip (cr);
 
       ge_cairo_set_color(cr, color1); 
-      cairo_arc(cr, centerX, centerY, radius + 1, 0,  2*M_PI);
+      cairo_arc(cr, centerX, centerY, radius + 1, 0,  2*G_PI);
       cairo_fill(cr);
 
       cairo_restore(cr);
 
       ge_cairo_set_color(cr, color3);
-      cairo_arc(cr, centerX, centerY, radius, 0, 2 * M_PI);
+      cairo_arc(cr, centerX, centerY, radius, 0, 2 * G_PI);
       cairo_fill(cr);
 
       break;
@@ -921,6 +859,7 @@ thinice_style_draw_shadow_gap(GtkStyle * style,
 
 	CairoColor              *color1 = NULL, *color2 = NULL;
 	cairo_t *cr;
+	gint start;
 
 	CHECK_ARGS
 	SANITIZE_SIZE
@@ -946,99 +885,30 @@ thinice_style_draw_shadow_gap(GtkStyle * style,
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
+	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
+	cairo_rectangle (cr, x, y, width, height);
+	
 	switch (gap_side) {
         case GTK_POS_TOP:
-		if (gap_x > 0) {
-			ge_cairo_line (cr, color1, 
-				       x, y, 
-				       x + gap_x, y);
-		}
-		if ((width - (gap_x + gap_width)) > 0) {
-			ge_cairo_line (cr, color1, 
-				       x + gap_x + gap_width - 1, y,
-				       x + width - 1, y);
-		}
-		ge_cairo_line (cr, color1, 
-			       x, y, 
-			       x, y + height - 1);
-		ge_cairo_line (cr, color2,
-			       x + width - 1, y,
-			       x + width - 1, y + height - 1);
-		ge_cairo_line (cr, color2,
-			       x, y + height - 1,
-			       x + width - 1, y + height - 1);
+        	start = MAX (1, gap_x + 1);
+        	cairo_rectangle (cr, x + start, y, MIN(width-1, gap_x+gap_width - 1) - start, 1);
 		break;
         case GTK_POS_BOTTOM:
-		ge_cairo_line (cr, color1,
-			       x, y,
-			       x + width - 1, y);
-		ge_cairo_line (cr, color1, 
-			       x, y, 
-			       x, y + height - 1);
-		ge_cairo_line (cr, color2,
-			       x + width - 1, y,
-			       x + width - 1, y + height - 1);
-
-		if (gap_x > 0) {
-			ge_cairo_line (cr, color2, 
-				       x, y + height - 1, 
-				       x + gap_x, y + height - 1);
-		}
-		if ((width - (gap_x + gap_width)) > 0) {
-			ge_cairo_line (cr, color2, 
-				       x + gap_x + gap_width - 1, y + height - 1,
-				       x + width - 1, y + height - 1);
-		}
-		
+        	start = MAX (1, gap_x + 1);
+        	cairo_rectangle (cr, x + start, y + height - 1, MIN(width-1, gap_x+gap_width - 1) - start, 1);
 		break;
         case GTK_POS_LEFT:
-		ge_cairo_line (cr, color1,
-			       x, y,
-			       x + width - 1, y);
-		if (gap_x > 0) {
-			ge_cairo_line (cr, color1, 
-				       x, y,
-				       x, y + gap_x);
-		}
-		if ((height - (gap_x + gap_width)) > 0) {
-			ge_cairo_line (cr, color1, 
-				       x, y + gap_x + gap_width - 1,
-				       x, y + height - 1);
-		}
-
-		ge_cairo_line (cr, color2,
-			       x + width - 1, y,
-			       x + width - 1, y + height - 1);
-		ge_cairo_line (cr, color2,
-			       x, y + height - 1,
-			       x + width - 1, y + height - 1);
+        	start = MAX (1, gap_x + 1);
+        	cairo_rectangle (cr, x, y + start, 1, MIN(width-1, gap_x+gap_width - 1) - start);
 		break;
         case GTK_POS_RIGHT:
-		ge_cairo_line (cr, color1,
-			       x, y,
-			       x + width - 1, y);
-		ge_cairo_line (cr, color1, 
-			       x, y, 
-			       x, y + height - 1);
-
-
-		if (gap_x > 0) {
-			ge_cairo_line (cr, color2, 
-				       x + width - 1, y,
-				       x + width - 1, y + gap_x);
-		}
-		if ((height - (gap_x + gap_width)) > 0) {
-			ge_cairo_line (cr, color2, 
-				       x + width - 1, y + gap_x + gap_width - 1,
-				       x + width - 1, y + height - 1);
-		}
-
-		ge_cairo_line (cr, color2,
-			       x, y + height - 1,
-			       x + width - 1, y + height - 1);
-
+        	start = MAX (1, gap_x + 1);
+        	cairo_rectangle (cr, x + width - 1, y + start, 1, MIN(width-1, gap_x+gap_width - 1) - start);
+        	break;
 	}
-
+	cairo_clip (cr);
+	cairo_new_path (cr);
+	ge_cairo_simple_border (cr, color1, color2, x, y, width, height, FALSE);
 	cairo_destroy(cr);
 }
 
@@ -1074,15 +944,62 @@ thinice_style_draw_box_gap(GtkStyle * style,
 static void
 thinice_style_draw_extension(DRAW_ARGS, GtkPositionType gap_side)
 {
-	switch (gap_side)
+	ThiniceStyle *thinice_style = THINICE_STYLE (style);
+
+	CairoColor *background, *light, *dark;
+	cairo_t *canvas;
+
+	CHECK_ARGS
+
+	SANITIZE_SIZE
+
+	background = &thinice_style->color_cube.bg[state_type];
+	light = &thinice_style->color_cube.light[state_type];
+	dark = &thinice_style->color_cube.dark[state_type];
+
+	canvas = ge_gdk_drawable_to_cairo (window, area);
+
+	if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
 	{
-		case GTK_POS_TOP: gap_side = GTK_POS_BOTTOM; break;
-		case GTK_POS_BOTTOM: gap_side = GTK_POS_TOP; break;
-		case GTK_POS_LEFT: gap_side = GTK_POS_RIGHT; break;
-		case GTK_POS_RIGHT: gap_side = GTK_POS_LEFT; break;
+		ge_cairo_set_color(canvas, background);
+
+		cairo_rectangle(canvas, x, y, width, height);
+
+		cairo_fill(canvas);
+	}
+	else
+	{
+		gtk_style_apply_default_background(style, window,
+				widget && !GTK_WIDGET_NO_WINDOW(widget),
+				state_type, area, x, y, width, height);
 	}
 
-	thinice_tab(DRAW_VARS, gap_side);
+	cairo_rectangle (canvas, x, y, width, height);
+	cairo_clip (canvas);
+
+	switch(gap_side)
+	{
+		case GTK_POS_TOP:
+			y--;
+			height++;
+		break;
+
+		case GTK_POS_LEFT:
+			x--;
+			width++;
+		break;
+
+		case GTK_POS_BOTTOM:
+			height++;
+		break;
+
+		case GTK_POS_RIGHT:
+			width++;
+		break;
+	}
+
+	ge_cairo_simple_border (canvas, light, dark, x, y, width, height, FALSE);
+	cairo_destroy(canvas);
 }
 
 static void
@@ -1147,9 +1064,9 @@ thinice_style_draw_slider(GtkStyle * style,
           int chopoff;
 
           if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-            chopoff = max(0, min(6, width-SMALLEST_HANDLE));
+            chopoff = CLAMP(width-SMALLEST_HANDLE, 0, 6);
           } else {
-            chopoff = max(0, min(6, height-SMALLEST_HANDLE));
+            chopoff = CLAMP(height-SMALLEST_HANDLE, 0, 6);
           }
 
           pointsh[0].x = x;                  pointsh[0].y = y+height-1;
@@ -1408,77 +1325,6 @@ thinice_style_draw_handle(GtkStyle * style,
     cairo_destroy(cr);
 }
 
-static void
-thinice_tab(DRAW_ARGS, gint orientation)
-{
-	ThiniceStyle *thinice_style = THINICE_STYLE (style);
-
-  CairoColor *light, *dark;
-  cairo_t *cr;
-
-  CHECK_ARGS
-
-  SANITIZE_SIZE
-
-  light = &thinice_style->color_cube.light[state_type];
-  dark = &thinice_style->color_cube.dark[state_type];
-
-  cr = ge_gdk_drawable_to_cairo (window, area);
-
-  if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
-    {
-      ge_cairo_set_color(cr, &thinice_style->color_cube.bg[state_type]);
-
-      cairo_rectangle(cr, x, y, width - 1, height - 1);
-
-      cairo_fill(cr);
-    }
-  else
-    {
-      gtk_style_apply_default_background(style, window,
-                                         widget && !GTK_WIDGET_NO_WINDOW(widget),
-                                         state_type, area, x, y, width, height);
-    }
-
-  switch(orientation)
-    {
-    case GTK_POS_TOP:
-      ge_cairo_line(cr, light,
-                    x, y + height, x, y);
-      ge_cairo_line(cr, light,
-                    x, y, x + width - 2, y);
-      ge_cairo_line(cr, dark,
-                    x + width - 2, y, x + width - 2, y + height - 1);
-      break;
-    case GTK_POS_BOTTOM:
-      ge_cairo_line(cr, light,
-                    x, y - 1, x, y + height - 1);
-      ge_cairo_line(cr, dark,
-                    x, y + height - 1, x + width - 2, y + height - 1);
-      ge_cairo_line(cr, dark,
-                    x + width - 2, y + height - 1, x + width - 2, y);
-      break;
-    case GTK_POS_LEFT:
-      ge_cairo_line(cr, light,
-                    x, y + height - 1, x, y);
-      ge_cairo_line(cr, light,
-                    x, y, x + width, y);
-      ge_cairo_line(cr, dark,
-                    x, y + height - 1, x + width - 1, y + height - 1);
-      break;
-    case GTK_POS_RIGHT:
-      ge_cairo_line(cr, light,
-                    x - 1, y, x + width - 1, y);
-      ge_cairo_line(cr, dark,
-                    x + width - 1, y, x + width - 1, y + height - 1);
-      ge_cairo_line(cr, dark,
-                    x, y + height - 1, x + width - 1, y + height - 1);
-      break;
-    }
-
-    cairo_destroy(cr);
-}
-
 GType thinice_type_style = 0;
 
 void
